/* 
OpenChrony v1.2
Originally written by Luke Johnson
Feel free modify/redistribute/sell/whatever - I don't care
Questions, concerns, and suggestions can be sent to: lukej11@gmail.com
*/

// Libraries, Display Definitions //
#include <Button.h>
#include <Wire.h>
#include <Adafruit_GFX.h>
#include <Adafruit_LEDBackpack.h>
Adafruit_7segment matrix = Adafruit_7segment();

// Pin Assignments //
const int gate1Pin = 2;
const int gate2Pin = 3;
const int greenLedPin = 6;
const int yellowLedPin = 7;
const int redLedPin = 8;
const int gate1LedPin = 9;
const int gate2LedPin = 10;
const int rstButtPin = 11;
const int modeSwitchPin = 12;

// Timing Variables //
unsigned long startT;
unsigned long stopT;
unsigned long elapsedT;

// Boolean Variables //
bool shoot = true;

// Buttons //
Button rstButt = Button(rstButtPin,PULLUP);

// Sensor Reading Function //
void shooting() {
  for (int i=0 ; i < 5 ; i++) {
    matrix.writeDigitRaw( i, B10000000 ); // write dots
  } 
  matrix.writeDisplay();
  digitalWrite(greenLedPin, HIGH);
  while (digitalRead(gate1Pin) == LOW) {
    // wait for projectile to enter first IR gate
  }
  startT = micros(); // record time when projectile passes first gate
  while (digitalRead(gate2Pin) == LOW) { 
    // wait for projectile to enter second IR gate
  }
  stopT = micros(); // record time when projectile passes second gate
  elapsedT = stopT - startT; // calculate difference
  shoot = false; // flag that says data aquisition has finished
  digitalWrite(greenLedPin, LOW);
}

// Troubleshoot Sensor Function //
void adjustPot() {
  if (digitalRead(gate1Pin) == LOW && digitalRead(gate2Pin) == LOW) {
    digitalWrite(gate1LedPin, LOW);
    digitalWrite(gate2LedPin, LOW);
  }
  if (digitalRead(gate1Pin) == HIGH && digitalRead(gate2Pin) == LOW) {
    digitalWrite(gate1LedPin, HIGH);
    digitalWrite(gate2LedPin, LOW);
  }
  if (digitalRead(gate1Pin) == LOW && digitalRead(gate2Pin) == HIGH) {
    digitalWrite(gate1LedPin, LOW);
    digitalWrite(gate2LedPin, HIGH);
  }
  if (digitalRead(gate1Pin) == HIGH && digitalRead(gate2Pin) == HIGH) {
    digitalWrite(gate1LedPin, HIGH);
    digitalWrite(gate2LedPin, HIGH);
  } 
}

// FPS Calculations //
int FPSmath(unsigned long lTime) {
  const float sensorDist = 0.5 * 1e6; // feet * 10e6  NOTE: if changing distance between IR gates, change the "0.5" to whatever your new distance is (in feet)
  float fTime = (float)lTime;
  float fFPS = round(sensorDist / fTime); // (feet * 10e6)/(seconds * 10e-6) = feet/second
  int iFPS = (int)fFPS;
  return iFPS;
}

// Setup Function //
void setup() {
  matrix.begin(0x70);
  Serial.begin(9600);
  pinMode(gate1Pin, INPUT_PULLUP);
  pinMode(gate2Pin, INPUT_PULLUP);
  pinMode(greenLedPin, OUTPUT);
  pinMode(yellowLedPin, OUTPUT);
  pinMode(redLedPin, OUTPUT);
  pinMode(gate1LedPin, OUTPUT);
  pinMode(gate2LedPin, OUTPUT);
  pinMode(modeSwitchPin, INPUT_PULLUP);
  for (int i=0 ; i < 4 ; i++) { // clear display
      matrix.writeDigitRaw( i, 0 );
    } 
    matrix.writeDisplay();
}

// Main Loop //
void loop() {
  int FPS4disp = 0;
  shooting();
  while (shoot == false) {
    digitalWrite(redLedPin, HIGH);
    FPS4disp = FPSmath(elapsedT);
    matrix.print(FPS4disp);
    matrix.writeDisplay();
    if (rstButt.uniquePress()) {
      startT = 0;
      stopT = 0; 
      digitalWrite(redLedPin, LOW);
      shoot = true;
    }
    while (digitalRead(modeSwitchPin) == HIGH) {
      digitalWrite(redLedPin, LOW);
      digitalWrite(yellowLedPin, HIGH);
      adjustPot();
    }
    digitalWrite(yellowLedPin, LOW);
  }

  digitalWrite(gate1LedPin, LOW);
  digitalWrite(gate2LedPin, LOW);
  
}
